.\" Copyright (c) 1983, 1991, 1993
.\"	The Regents of the University of California.  All rights reserved.
.\"
.\" Redistribution and use in source and binary forms, with or without
.\" modification, are permitted provided that the following conditions
.\" are met:
.\" 1. Redistributions of source code must retain the above copyright
.\"    notice, this list of conditions and the following disclaimer.
.\" 2. Redistributions in binary form must reproduce the above copyright
.\"    notice, this list of conditions and the following disclaimer in the
.\"    documentation and/or other materials provided with the distribution.
.\" 4. Neither the name of the University nor the names of its contributors
.\"    may be used to endorse or promote products derived from this software
.\"    without specific prior written permission.
.\"
.\" THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
.\" ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
.\" IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
.\" ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
.\" FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
.\" DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
.\" OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
.\" HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
.\" LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
.\" OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
.\" SUCH DAMAGE.
.\"
.\"     @(#)scandir.3	8.1 (Berkeley) 6/4/93
.\" $FreeBSD: src/lib/libc/gen/scandir.3,v 1.9 2007/01/09 00:27:55 imp Exp $
.\"
.Dd May 20, 2008
.Dt SCANDIR 3
.Os
.Sh NAME
.Nm scandir ,
#ifdef UNIFDEF_BLOCKS
.Nm scandir_b ,
#endif
.Nm alphasort
.Nd scan a directory
.Sh SYNOPSIS
.In sys/types.h
.In dirent.h
.Ft int
.Fn scandir "const char *dirname" "struct dirent ***namelist" "int \\*(lp*select\\*(rp\\*(lpconst struct dirent *\\*(rp" "int \\*(lp*compar\\*(rp\\*(lpconst struct dirent **, const struct dirent **\\*(rp"
.Ft int
.Fn alphasort "const struct dirent **d1" "const struct dirent **d2"
#ifdef UNIFDEF_BLOCKS
.Ft int
.Fn scandir_b "const char *dirname" "struct dirent ***namelist" "int \\*(lp^select\\*(rp\\*(lpconst struct dirent *\\*(rp" "int \\*(lp^compar\\*(rp\\*(lpconst struct dirent **, const struct dirent **\\*(rp"
#endif
.Sh DESCRIPTION
The
.Fn scandir
function
reads the directory
.Fa dirname
and builds an array of pointers to directory
entries using
.Xr malloc 3 .
It returns the number of entries in the array.
A pointer to the array of directory entries is stored in the location
referenced by
.Fa namelist 
(even if the number of entries is 0).
.Pp
The
.Fa select
argument is a pointer to a user supplied subroutine which is called by
.Fn scandir
to select which entries are to be included in the array.
The select routine is passed a
pointer to a directory entry and should return a non-zero
value if the directory entry is to be included in the array.
If
.Fa select
is null, then all the directory entries will be included.
.Pp
The
.Fa compar
argument is a pointer to a user supplied subroutine which is passed to
.Xr qsort 3
to sort the completed array.
If this pointer is null, the array is not sorted.
.Pp
The
.Fn alphasort
function
is a routine which can be used for the
.Fa compar
argument to sort the array alphabetically.
.Pp
The memory allocated for the array can be deallocated with
.Xr free 3 ,
by freeing each pointer in the array and then the array itself.
#ifdef UNIFDEF_BLOCKS
.Pp
The
.Fn scandir_b
function works the same way as the
.Fn scandir
function, except that
.Fa select
and
.Fa compar
are blocks instead of subroutines.
#endif
.Sh DIAGNOSTICS
Returns \-1 if the directory cannot be opened for reading or if
.Xr malloc 3
cannot allocate enough memory to hold all the data structures.
.Sh SEE ALSO
.Xr directory 3 ,
.Xr malloc 3 ,
.Xr qsort 3 ,
.Xr dir 5
.Sh HISTORY
The
.Fn scandir
and
.Fn alphasort
functions appeared in
.Bx 4.2 .
#ifdef UNIFDEF_BLOCKS
The
.Fn scandir_b
function appeared in Mac OS X 10.6.
#endif
